#!/usr/bin/perl

use strict;
use Test::More;

my $nbtest = 0;
my $res;
my $exp;

####
#### Basic tests: 3
####

# BEGIN { use_ok(...)} are now at the end, after issuing the plan

$FDN::Common::Config::CONFIG_ROOT = './t/';
our $dbh = do_connect('test');
is ( ref($dbh), 'HASH', 'do_connect' );
BEGIN { $nbtest += 1; }

####
#### Init db -> 36
####

my $sql = \&FDN::Sql::DB_Access::sql_transaction;
my @queries = (
# /1
"drop table if exists SEQ",

# /2
"create table SEQ (                                        ".
"    SEQ_NAME varchar(20) UNIQUE NOT NULL default ''      ,".
"    SEQ_VALUE int(11) NOT NULL default '0'               ,".
"    SEQ_MINVALUE int(11) NOT NULL default '0'            ,".
"    SEQ_MAXVALUE int(11) NOT NULL default '0'            ,".
"    SEQ_INC int(11) NOT NULL default '0'                 ,".
"    SEQ_CYCLE ENUM('-1','0','1') NOT NULL default '-1'   ,".
"    primary key  (SEQ_NAME))",

# /3
"drop table if exists TEST",

# /4
"create table TEST (                                       ".
"    TEST_ID int(11) UNIQUE NOT NULL default '0'          ,".
"    TEST_NOM varchar(70)                                 ,".
"    TEST_ETAT enum('oui','non','autre','encore')         ,".
"    TEST_DTMOD timestamp                                 ,".
"    TEST_DTIN  timestamp                                 ,".
"    TEST_DTOUT timestamp                                 ,".
"    primary key(TEST_ID)                                 ,".
"    index(TEST_NOM)                                      ,".
"    index(TEST_ETAT) )",

# /5
"replace into SEQ values('TEST_ID',0,0,99999999,1,'0')",

# /6
"drop table if exists TESTSELF",

# /7
"create table TESTSELF (                                   ".
"    TEST_ID int(11) NOT NULL default '0'                 ,".
"    SELF_ID int(11) NOT NULL default '0'                 ,".
"    TESTSELF_NOM varchar(70) )",

# /8
"drop table if exists PROPTEST",

# /9
"create table PROPTEST (                                   ".
"    PROPTEST_ID int(11) UNIQUE NOT NULL default '0'      ,".
"    TEST_ID int(11) default NULL                         ,".
"    PROPTEST_NOM varchar(70) default NULL                ,".
"    PROPTEST_VAL text                                    ,".
"    PROPTEST_PAR int(11) default NULL                    ,".
"    primary key(PROPTEST_ID)                             ,".
"    index(TEST_ID) )",

# /10
"replace into SEQ values('PROPTEST_ID',0,0,99999999,1,'0')",

# /11
"drop table if exists A",

# /12
"create table A (                                          ".
"    A_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    A_NOM varchar(70)                                    ,".
"    primary key(A_ID) )",

# /13
"replace into SEQ values('A_ID',0,0,99999999,1,'0')",

# /14
"drop table if exists B",

# /15
"create table B (                                          ".
"    B_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    B_NOM varchar(70)                                    ,".
"    primary key(B_ID) )",

# /16
"replace into SEQ values('B_ID',0,0,99999999,1,'0')",

# /17
"drop table if exists AB",

# /18
"create table AB (                                         ".
"    A_ID int(11) NOT NULL default '0'                    ,".
"    B_ID int(11) NOT NULL default '0'                    ,".
"    AB_NOM varchar(70)                                   ,".
"    AB_ETAT enum('oui','non','autre','encore')           ,".
"    index(A_ID)                                          ,".
"    index(B_ID)                                          ,".
"    index(A_ID,B_ID) )",

# /19
"drop table if exists C",

# /20
"create table C (                                          ".
"    C_ID int(11) UNIQUE NOT NULL default '0'             ,".
"    C_NOM varchar(70)                                    ,".
"    primary key(C_ID) )",

# /21
"replace into SEQ values('C_ID',0,0,99999999,1,'0')",

# /22
"drop table if exists BC",

# /23
"create table BC (                                         ".
"    B_ID int(11) NOT NULL default '0'                    ,".
"    C_ID int(11) NOT NULL default '0'                    ,".
"    index(B_ID)                                          ,".
"    index(C_ID)                                          ,".
"    index(B_ID,C_ID) )",

# /24
"drop table if exists OTHER",

# /25
"create table OTHER (                                      ".
"    OTHER_ID int(11) UNIQUE NOT NULL default '0'         ,".
"    OTHER_NOM varchar(70)                                ,".
"    OTHER_FLD varchar(70)                                ,".
"    TEST_ID int(11) NOT NULL default '0'                 ,".
"    primary key(OTHER_ID)                                ,".
"    index(TEST_ID) )",

# /26
"replace into SEQ values('OTHER_ID',0,0,99999999,1,'0')",

# /27
"drop table if exists ROTATION",

# /28
"create table ROTATION (                                   ".
"    ROTATION_ID int(11) UNIQUE NOT NULL default '0'      ,".
"    ROTATION_TABLE varchar(70)                           ,".
"    ROTATION_DATE  timestamp                             ,".
"    ROTATION_DEST  varchar(70)                           ,".
"    primary key(ROTATION_ID)                             ,".
"    index(ROTATION_TABLE)                                ,".
"    index(ROTATION_DATE) )",

# /29
"replace into SEQ values('ROTATION_ID',0,0,99999999,1,'0')",

# /30
"drop table if exists LOG",

# /31
"drop table if exists LOG20050501020000",

# /32
"drop table if exists LOG20050601020000",

# /33
"drop table if exists LOG20050701020000",

# /34
"create table LOG (                                        ".
"    LOG_ID int(11) UNIQUE NOT NULL default '0'           ,".
"    LOG_DATE timestamp                                   ,".
"    LOG_INFO varchar(70)                                 ,".
"    A_ID int(11) NOT NULL default '-1'                   ,".
"    primary key(LOG_ID)                                  ,".
"    index(LOG_DATE) )",

# /35
"replace into SEQ values('LOG_ID',0,0,99999999,1,'0')",

# /36
"drop table if exists NOLOG",

# /37
"create table NOLOG (                                        ".
"    NOLOG_ID int(11) UNIQUE NOT NULL default '0'           ,".
"    NOLOG_DATE timestamp                                   ,".
"    NOLOG_INFO varchar(70)                                 ,".
"    primary key(NOLOG_ID)                                  ,".
"    index(NOLOG_DATE) )",

# /38
"replace into SEQ values('NOLOG_ID',0,0,99999999,1,'0')",

);

for my $q ( @queries ) {
	$res = &$sql($dbh,'create',$q);
	ok ( $res->{'ok'}, 'init db' );
	if ( ! $res->{'ok'} ) {
		die "Failed req. ".$q;
	}
}
BEGIN { $nbtest += 38; }

####
#### Insertions: 5
####

minimal_functions({
	'A' => [ 'NOM' ],
	'B' => [ 'NOM' ]
});

# /1
$res = new_a('toto');
is ( $res, 1, 'new_a');

# /2
$res = new_a('titi');
is ( $res, 2, 'new_a');

# /3
$res = new_a('titi');
is ( $res, 3, 'new_a');

# /4
$res = new_b('nom1');
is ( $res, 1, 'new_b');

# /5
$res = new_b('nom2');
is ( $res, 2, 'new_b');

BEGIN { $nbtest += 5; }


###minimal_functions({ # XXX On sait pas gerer les dates dans Basic2
###	'TEST' => [ 'NOM', 'ETAT', 'DTIN' ]
###});
###
#### /0
###$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test1'), 'TEST_ETAT='.dochar('oui'),
###		'TEST_DTIN='.dodate('now') );
###is ( $res, 1, '__new');
###
#### /0
###$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test2'), 'TEST_ETAT='.dochar('non'),
###		'TEST_DTIN='.dodate('now') );
###is ( $res, 2, '__new');
###

minimal_functions({
	'OTHER' => [ 'NOM', 'FLD', 'TEST_ID' ]
});
# /1
$res = new_other('autre1', 'val1', '2');
is ( $res, 1, 'new_other');

# /2
$res = new_other('autre2', 'val2', '2');
is ( $res, 2, 'new_other');

# /3
$res = new_other('autre3', 'val3', '1');
is ( $res, 3, 'new_other');

BEGIN { $nbtest += 3; }

#### /14
###$res = __set_self_relation($dbh, 'TEST', 1, 2, 'NOM='.dochar('selfrel') );
###ok ( $res, '__set_self_relation' );
###
#### /15
###$res = __set_relation($dbh, 'A', 'B', 3, 1, 'NOM='.dochar('type1') );
###ok ( $res, '__set_relation' );
###
#### /16
###$res = __set_relation($dbh, 'A', 'B', 3, 1, 'NOM='.dochar('type2') );
###ok ( $res, '__set_relation' );
###
#### /17
###$res = __set_relation($dbh, 'A', 'B', 3, 2, 'NOM='.dochar('type1') );
###ok ( $res, '__set_relation' );
###
#### /18
###$res = __set_relation($dbh, 'A', 'B', 4, 2, 'NOM='.dochar('type1') );
###ok ( $res, '__set_relation' );
###
#### /19
###$res = __new($dbh, 'TEST', 'TEST_NOM='.dochar('test3'), 'TEST_ETAT='.dochar('non'),
###		'TEST_DTIN='.dodate('now') );
###is ( $res, 3, '__new');
###
#### /20
###$res = __set_self_relation($dbh, 'TEST', 1, 3, 'NOM='.dochar('selfrel2') );
###ok ( $res, '__set_self_relation' );
###
#### /21
###$res = __new($dbh, 'C', 'NOM='.dochar('elemC-gen-1') );
###is ( $res, 1, '__new');
###
#### /22
###$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-1') );
###is ( $res, 2, '__new');
###
#### /23
###$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-2') );
###is ( $res, 3, '__new');
###
#### /24
###$res = __new($dbh, 'C', 'NOM='.dochar('elemC-spe-3') );
###is ( $res, 4, '__new');
###
#### /25
###$res = __set_relation($dbh, 'B', 'C', 1, 1 );
###ok ( $res, '__set_relation');
###
#### /26
###$res = __set_relation($dbh, 'B', 'C', 1, 2 );
###ok ( $res, '__set_relation');
###
#### /27
###$res = __set_relation($dbh, 'B', 'C', 2, 1 );
###ok ( $res, '__set_relation');
###
#### /28
###$res = __set_relation($dbh, 'B', 'C', 2, 3 );
###ok ( $res, '__set_relation');
###BEGIN { $nbtest += 5; }


####
#### Recherches simples: 38
####

$res = a_par_id(6);
is ( $res, undef, 'a_par_id' );
$res = other_par_id(1);
is ( ref($res), 'HASH', 'other_par_id');
is ( $res->{'OTHER_ID'}, '1', 'other_par_id');
is ( $res->{'OTHER_NOM'}, 'autre1', 'other_par_id');
is ( $res->{'OTHER_FLD'}, 'val1', 'other_par_id' );
is ( $res->{'TEST_ID'}, '2', 'other_par_id' );
BEGIN { $nbtest += 6; }


search_functions([
	'A_ID' => [ 'NOM' ],
	'OTHER_ID' => [ 'TEST_ID' ],
	'OTHER_ID' => [ 'NOM' ],
	'OTHER_ID' => [ 'TEST_ID', 'FLD' ]
]);

# /1
$res = a_id_par_nom('toto');
is ( $res, 1, 'a_id_par_nom' );
# /2
$res = [ a_id_par_nom('titi' ) ];
ok ( eq_set( $res, [2,3] ), 'a_id_par_nom' );
# /3
$res = other_id_par_nom('autre2');
is ( $res, 2, 'other_id_par_nom' );
# /4
$res = other_id_par_test_id(1);
is ( $res, 3, 'other_id_par_test_id' );
# /5-6
$res = [ other_id_par_test_id(2) ];
is ( $#$res, 1, 'other_id_par_test_id' );
ok ( eq_set($res, [1,2]), 'other_id_par_test_id');
# /7-8
$res = [ other_id_par_test_id_fld(2, 'val2') ];
is ( $#$res, 0, '__id_par_clef' );
ok ( eq_set($res, [2]), '__id_par_clef');
BEGIN { $nbtest += 8; }


search_functions([
	'OTHER' => [ 'NOM' ]
]);
$res = other_par_nom('autre1');
is ( ref($res), 'HASH', 'other_par_nom' );
is ( $res->{'OTHER_ID'}, 1, 'other_par_nom' );
is ( $res->{'OTHER_NOM'}, 'autre1', 'other_par_nom' );
is ( $res->{'OTHER_FLD'}, 'val1', 'other_par_nom' );
is ( $res->{'TEST_ID'}, 2, 'other_par_nom' );
BEGIN { $nbtest += 5; }


update_functions([
	'OTHER' => 'NOM'
]);

$exp = other_par_id(1);
$exp->{'OTHER_NOM'} = 'prout';
$res = set_other_nom(1,'prout');
ok ( $res, 'set_other_nom');
$res = other_par_id(1);
is_deeply($res, $exp, 'set_other_nom');
BEGIN { $nbtest += 2; }



BEGIN { plan tests => $nbtest + 2; } 
BEGIN { use_ok('FDN::Sql::Basic2'); }
BEGIN { use_ok('FDN::Sql::DBC'); }


